import { useState, useRef } from "react";
import { TypedUseSelectorHook, useDispatch, useSelector } from "react-redux";
import type { RootState, AppDispatch } from "./store";
import { clamp } from "lodash";

export const useAppDispatch: () => AppDispatch = useDispatch;
export const useAppSelector: TypedUseSelectorHook<RootState> = useSelector;

const invLerp = (a: number, b: number, x: number): number =>
  clamp((x - b) / (a - b), 0, 1);

interface Timer {
  segs: number;
  start: () => void;
  cancel: () => void;
}

export const useTimer = (length: number, onTimeout?: () => unknown): Timer => {
  const [segs, setSegs] = useState(0);
  const timeoutId = useRef<ReturnType<typeof setTimeout>>();

  const updateTimer = (startTime: number) => {
    const endTime = startTime + length;
    const currentTime = Date.now();
    const segs = invLerp(startTime, endTime, currentTime) * 5;
    setSegs(segs);
    if (segs > 0) {
      timeoutId.current = setTimeout(updateTimer, 100, startTime);
    } else if (onTimeout) {
      onTimeout();
    }
  };

  const start = () => {
    const startTime = Date.now();
    timeoutId.current = setTimeout(updateTimer, 100, startTime);
  };

  const cancel = () => {
    clearTimeout(timeoutId.current);
    setSegs(0);
  };

  return {
    segs,
    start,
    cancel,
  };
};
